/*************************************************** */
/* Rule Set Based Access Control                     */
/*                                                   */
/* Author and (c) 1999-2005: Amon Ott <ao@rsbac.org> */
/*                                                   */
/* Last modified: 29/Sep/2005                        */
/*************************************************** */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <grp.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/resource.h>
#include <rsbac/types.h>
#include <rsbac/syscalls.h>
#include <rsbac/error.h>
#include "nls.h"
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

  char * progname;
  rsbac_list_repl_partner_number_t id;
  __u32 crypt_key_len = 0;
  __u32 ip;
  __u16 port = 555;
  __u32 flags;
  rsbac_time_t ttl = 0;
  int verbose = 0;
  rsbac_list_ta_number_t ta_number = 0;

void use(void)
    {
      printf(gettext("%s (RSBAC %s)\n***\n"), progname, VERSION);
      printf(gettext("Use: %s [flags] id name ip\n"), progname);
      printf(gettext(" id   = partner id to use,\n"));
      printf(gettext(" name = name of replication partner,\n"));
      printf(gettext(" ip   = IP address of replication partner,\n"));
      printf(gettext(" -C algo = name of encryption algorithm (default is none),\n"));
      printf(gettext(" -K key = encryption key in hex notation,\n"));
      printf(gettext(" -p port = use port instead of default 555,\n"));
      printf(gettext(" -a = activate immediately,\n"));
      printf(gettext(" -m = partner is a master for this host,\n"));
      printf(gettext(" -s = partner is a slave of this host,\n"));
      printf(gettext(" -t = set relative time-to-live in secs\n"));
      printf(gettext(" -T = set absolute time-to-live in secs\n"));
      printf(gettext(" -D = set relative time-to-live in days\n"));
      printf(gettext(" -N ta = transaction number (default = value of RSBAC_TA, if set, or 0)\n"));
    }

int key_read(char * to, char * from)
  {
    char * f = from;
    char * t = to;
    char   tmp[3];
    int i;
    int res = 0;

    if(strlen(from) > RSBAC_LIST_REPL_CRYPTKEY_LEN * 2)
      {
        fprintf(stderr, "Key is too long!\n");
        return -RSBAC_EINVALIDVALUE;
      }
    tmp[2] = 0;
    while(f[0] && f[1])
      {
        tmp[0] = f[0];
        tmp[1] = f[1];
        i = strtoul(tmp, 0, 16);
        if(i < 0 || i > 255)
          return -RSBAC_EINVALIDVALUE;
        *t = i;
        t++;
        f += 2;
        res++;
      }
    return res;
  }

int env_init(void)
{
	int res;

	struct rlimit rlimit;

	rlimit.rlim_cur = 0;
	rlimit.rlim_max = 0;
	res = setrlimit(RLIMIT_CORE, &rlimit);
	if (res)
		fprintf(stderr, gettext("Warning: unable to set resource limits.\n"));	
	
	return res;
}

int main(int argc, char ** argv)
{
  int res = 0;
  struct rsbac_list_repl_partner_entry_t entry;

  env_init();
  locale_init();

  progname = argv[0];
  memset(&entry, 0, sizeof(entry));
  {
    char * env = getenv("RSBAC_TA");

    if(env)
      ta_number = strtoul(env,0,0);
  }
  res = mlock(&entry, sizeof(entry));
  if (res)
	  fprintf(stderr, gettext("Warning: replication parter data was not locked into physical memory.\n"));
  while((argc > 1) && (argv[1][0] == '-'))
    {
      char * pos = argv[1];
      pos++;
      while(*pos)
        {
          switch(*pos)
            {
              case 'h':
                use();
                return 0;
              case 'v':
                verbose++;
                break;
              case 'a':
                flags |= REPL_flag_active;
                break;
              case 'm':
                flags |= REPL_flag_master;
                break;
              case 's':
                flags |= REPL_flag_slave;
                break;
              case 'p':
                if(argc > 2)
                  {
                    port = strtoul(argv[2],0,0);
                    argc--;
                    argv++;
                  }
                else
                  fprintf(stderr, gettext("%s: missing argument for parameter %c\n"), progname, *pos);
                break;
              case 'K':
                if(argc > 2)
                  {
                    res = key_read(entry.crypt_key, argv[2]);
                    error_exit(res);
                    entry.crypt_key_len = res;
                    argc--;
                    argv++;
                  }
                else
                  fprintf(stderr, gettext("%s: missing argument for parameter %c\n"), progname, *pos);
                break;
              case 'C':
                if(argc > 2)
                  {
                    strncpy(entry.crypt_algo, argv[2], RSBAC_LIST_REPL_CRYPTALGO_LEN);
                    entry.crypt_algo[RSBAC_LIST_REPL_CRYPTALGO_LEN - 1] = 0;
                    argc--;
                    argv++;
                  }
                else
                  fprintf(stderr, gettext("%s: missing argument for parameter %c\n"), progname, *pos);
                break;
              case 't':
                if(argc > 2)
                  {
                    ttl = strtoul(argv[2], 0, 10);
                    argc--;
                    argv++;
                  }
                else
                  fprintf(stderr, gettext("%s: missing ttl value for parameter %c\n"), progname, *pos);
                break;
              case 'D':
                if(argc > 2)
                  {
                    ttl = 86400 * strtoul(argv[2], 0, 10);
                    argc--;
                    argv++;
                  }
                else
                  fprintf(stderr, gettext("%s: missing ttl value for parameter %c\n"), progname, *pos);
                break;
              case 'T':
                if(argc > 2)
                  {
                    rsbac_time_t now = time(NULL);
                    ttl = strtoul(argv[2], 0, 10);
                    if(ttl > now)
                      {
                        ttl -= now;
                        argc--;
                        argv++;
                      }
                    else
                      {
                        fprintf(stderr,
                                gettext("%s: ttl value for parameter %c is in the past, exiting\n"), progname, *pos);
                        exit(1);
                      }
                  }
                else
                  fprintf(stderr, gettext("%s: missing ttl value for parameter %c\n"), progname, *pos);
                break;
              case 'N':
                if(argc > 2)
                  {
                    ta_number = strtoul(argv[2], 0, 10);
                    argc--;
                    argv++;
                  }
                else
                  {
                    fprintf(stderr, gettext("%s: missing transaction number value for parameter %c\n"), progname, *pos);
                    exit(1);
                  }
                break;

              default:
                fprintf(stderr, gettext("%s: unknown parameter %c\n"), progname, *pos);
                exit(1);
            }
          pos++;
        }
      argv++;
      argc--;
    }

  if (argc > 3)
    {
      int i;
      struct in_addr addr;

      id = strtoul(argv[1], 0, 0);
      if(!id)
        error_exit(-RSBAC_EINVALIDVALUE);
      strncpy(entry.name, argv[2], RSBAC_LIST_REPL_NAME_LEN);
      entry.name[RSBAC_LIST_REPL_NAME_LEN - 1] = 0;
      error_exit(inet_aton(argv[3], &addr));
      entry.ip_addr = addr.s_addr;
      entry.ip_port = htons(port);
      if(verbose)
        printf("Adding replication partner %u '%s' with ip address %u.%u.%u.%u\n",
               id, entry.name, NIPQUAD(entry.ip_addr) );
      res = rsbac_list_repl_add_partner(ta_number, id, &entry, ttl);
      error_exit(res);
      exit(0);
    }
  else
    {
      use();
      return 1;
    }
  return (res);
}
